# updated_on_server, updated_on_local, conflicts, deleted_conflicts, deleted_on_local, deleted_on_remote
class CommitStatus:
    UPDATED_ON_SERVER = "updated_on_server"
    UPDATED_ON_LOCAL = "updated_on_local"
    CONFLICTS = "conflicts"
    DELETED_CONFLICTS = "deleted_conflicts"
    DELETED_ON_LOCAL = "deleted_on_local"
    DELETED_ON_REMOTE = "deleted_on_remote"
    COMMIT_SHA1 = "commit_sha1"
